// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CORE_DATABASE_H
#define CORE_DATABASE_H

#include <map>
#include <string>

#include <QString>
#include <QStandardPaths>
#include <QDir>
#include <lmdb.h>

class DataBase
{
    class _Table;
    template<class T>
    class Serializer;
public:
    template <class K, class V>
    class Table;

    DataBase(const QString& name, uint16_t mapSize = 10);
    ~DataBase();

    void open();
    void close();
    QString getName() const;

    template <class K, class V>
    Table<K, V>* addTable(const QString& name);

public:
    //exceptions
    class Directory;
    class Closed;
    class Opened;
    class NotFound;
    class Exist;
    class Unknown;

private:
    std::string name;
    bool opened;
    uint16_t size;
    MDB_env* environment;
    std::map<std::string, _Table*> tables;
};

#include "exceptions.h"

template <class K, class V>
DataBase::Table<K, V>* DataBase::addTable(const QString& p_name) {
    std::string nm = p_name.toStdString();
    if (opened) {
        throw Opened(name, nm);
    }
    DataBase::Table<K, V>* table = new DataBase::Table<K, V>(nm, this);
    tables.insert(std::make_pair(nm, (_Table*)table));
    return table;
}

#endif // CORE_DATABASE_H
